export class AWXJobDetails {
  constructor(job) {
    this.job = job;
  }

  isRunning() {
    return !this.job?.finished;
  }

  isFinished() {
    return !this.isRunning();
  }

  wasSuccessful() {
    return this.job?.status === "successful";
  }
}

export class DeploymentInfoDetails {
  constructor(deployment) {
    this.deployment = deployment;
  }

  hasError() {
    return this.deployment?.status.toLowerCase() === "error";
  }

  type() {
    if (this.deployment) {
      return this.deployment.undeployment ? "undeployment" : "deployment";
    } else {
      return null;
    }
  }
}

const jobMessages = {
  new: "job has just been created",
  pending: "job dependencies are running",
  waiting: "job is queued on job server",
  running: "job is running",
  failed: "job failed",
  successful: "job was successful",
  error: "job had an error",
  canceled: "job was canceled"
};
export class DeploymentStatus {
  constructor(deployment, deploymentJob) {
    this.deployment = deployment;
    this.deploymentJob = deploymentJob;
    this.deploymentHelper = new DeploymentInfoDetails(deployment);
    this.jobHelper = new AWXJobDetails(deploymentJob);
  }

  isFinished() {
    return this.deploymentHelper.hasError() || this.jobHelper.isFinished();
  }

  wasSuccessful() {
    return this.jobHelper.wasSuccessful();
  }

  status() {
    const entity = this.deploymentJob ?? this.deployment;
    const status = entity.status.toLowerCase();
    if (["error, failed, canceled"].includes(status)) {
      return "failed";
    } else if (["queued", "waiting"].includes(status)) {
      return "queued";
    } else if (["new", "pending"].includes(status)) {
      return "pending";
    } else {
      return status;
    }
  }

  message() {
    const joblessMessages = {
      queued: "is queued for job creation",
      error: "job could not be created",
      job: "has an associated job"
    };

    const stem = `The ${this.deploymentHelper.type()} `;
    const info = this.deploymentJob
      ? jobMessages[this.deploymentJob.status.toLowerCase()]
      : joblessMessages[this.deployment.status.toLowerCase()];
    const message = stem + info;
    return message;
  }

  severity() {
    const entity = this.deploymentJob ?? this.deployment;
    const status = entity.status;
    if (["failed", "error", "canceled"].includes(status)) {
      return "error";
    } else if ("successful".includes(status)) {
      return "success";
    } else {
      return "info";
    }
  }
}

export class AWXCommandDetails {
  constructor(job) {
    this.job = job;
  }

  notFinished() {
    return !this.isFinished();
  }

  isFinished() {
    return ["failed", "canceled", "successful"].includes(this.job?.status);
  }

  wasSuccessful() {
    return this.job?.status === "successful";
  }

  wasFailed() {
    return this.job?.status === "failed";
  }

  isRunning() {
    return this.job?.status === "running";
  }
}

export class AdHocCommand {
  constructor(command) {
    this.command = command;
  }

  isRunning() {
    return !this.isFinished();
  }

  isFinished() {
    return ["failed", "canceled", "successful"].includes(this.command?.status);
  }

  wasSuccessful() {
    return this.job?.status === "successful";
  }

  message() {
    const message = `The ${this.command.type} command ${
      jobMessages[this.command?.status]
    }`;
    return message;
  }
}

export class CommandStatus {
  constructor(command, commandJob) {
    this.command = command;
    this.job = commandJob;
    this.commandHelper = new AdHocCommand(command);
    this.jobHelper = new AWXCommandDetails(commandJob);
  }

  message() {
    const message = `The ${this.command.type} command ${
      jobMessages[this.command.status]
    }`;
    return message;
  }
}

export class IocActiveDeployment {
  constructor(activeDeployment) {
    this.activeDeployment = activeDeployment;
  }

  failedDeployment() {
    return (
      this.activeDeployment &&
      "successful".localeCompare(this.activeDeployment?.jobStatus, undefined, {
        sensitivity: "base"
      }) !== 0
    );
  }

  calculateNotDeployedText(deployedText) {
    if (this.activeDeployment) {
      return deployedText;
    }

    return "---";
  }

  hasDeployment() {
    return (
      this.activeDeployment !== null && this.activeDeployment !== "undefined"
    );
  }
}
