/* eslint-disable react-hooks/exhaustive-deps */
/**
 * SwaggerApi
 * Swagger API, fetching data
 * and providing apiContext and userContext
 */
import React, { useCallback, useRef } from "react";
import SwaggerClient from "swagger-client";
import { createContext, useContext, useEffect, useState } from "react";
import { CircularProgress } from "@material-ui/core";
import { useCustomSnackbar } from "../components/common/snackbar/Snackbar";

const createSwaggerClient = async () => {
  const swaggerOptions = {
    url: `${window.SERVER_ADDRESS}${window.API_BASE_ENDPOINT}`,
    requestInterceptor: (req) => {
      // if (user) req.headers["Authorization"] = `Bearer ${user.token}`;
      req.headers["Content-Type"] = "application/json";
      req.headers.accept = "application/json";
      window.req = req;
      return req;
    },
    responseInterceptor: (res) => {
      return res;
    }
  };

  const swagger = await SwaggerClient(swaggerOptions);
  // update the server url to be our intended server
  swagger.spec.servers[0].url = `${window.SERVER_ADDRESS}`;
  window.swagger = swagger;
  return swagger;
};

export function useAPI() {
  const create = useCallback(createSwaggerClient, []);
  return useAsync({ fcn: create });
}

export const apiContext = createContext(null);
export const userContext = createContext({
  user: null,
  userRoles: [],
  getUserRoles: () => {},
  login: () => {},
  logout: () => {}
});

export function UserProvider({ children }) {
  function onLoginError(message) {
    setLoginError(message);
  }

  const [user, getUser, resetUser, userLoading] = useUser();
  const [loginError, setLoginError] = useState();
  const [loginResponse, loginFcn /* reset*/, , loginLoading] =
    useLogin(onLoginError);
  const [logoutFcn, logoutLoading] = useLogout();
  const [initialized, setInitialized] = useState(false);

  const resetLoginError = useCallback(
    () => setLoginError(null),
    [setLoginError]
  );

  console.log("rendering UserProvider");
  useEffect(getUser, [loginResponse, getUser]);

  const logout = useCallback(async () => {
    console.log("Logging out!");
    resetUser();
    logoutFcn();
  }, [resetUser, logoutFcn]);

  const login = useCallback(
    (username, password) => {
      console.log("Logging in!");
      loginFcn(username, password);
    },
    [loginFcn]
  );

  const loading = Boolean(loginLoading || userLoading || logoutLoading);

  const createValue = useCallback(() => {
    return {
      user: user?.userInfo,
      userRoles: user?.userRoles,
      getUser,
      login,
      loginError,
      resetLoginError,
      logout
    };
  }, [
    user?.userInfo,
    user?.userRoles,
    getUser,
    login,
    loginError,
    resetLoginError,
    logout
  ]);

  const [value, setValue] = useState(createValue());

  useEffect(() => {
    if (!loading) {
      setInitialized(true);
    }
  }, [loading]);

  useEffect(() => {
    if (!loading) {
      setValue(createValue());
    }
  }, [loading, setValue, createValue]);

  return (
    initialized && (
      <userContext.Provider value={value}>{children}</userContext.Provider>
    )
  );
}

export function APIProvider({ children }) {
  const [api] = useAPI();

  return api ? (
    <apiContext.Provider value={api}>{children}</apiContext.Provider>
  ) : (
    <CircularProgress />
  );
}

export function useAPIErrorHandler(onError) {
  const { logout } = useContext(userContext);
  const showError = useCustomSnackbar();

  function errorHandler(e) {
    const { response, message } = e;
    const { obj, status } = response ?? { obj: {}, status: "" };

    // Always log out if 401
    if (status === 401) {
      logout();
    }

    if (onError) {
      const errorNotHandled = onError(obj?.description ?? message, status);
      if (errorNotHandled) {
        showError(obj?.description ?? message);
      }
    } else {
      showError(obj?.description ?? message);
    }
  }

  return useCallback(errorHandler, [logout]);
}

export function useQueuedResponse({
  init = null,
  onError = null,
  initLoading = false
}) {
  const queueRef = useRef([]);
  const busyRef = useRef(false);
  const [response, setResponse] = useState(init);
  const [loading, setLoading] = useState(initLoading);
  const errorHandler = useAPIErrorHandler(onError);
  const MAX_QUEUE_LENGTH = 10;

  const processQueue = useCallback(async () => {
    if (busyRef.current) {
      return;
    }

    busyRef.current = true;
    setLoading(true);

    let responses = [];
    const queue = queueRef.current;
    while (responses.length !== queue.length) {
      responses = await Promise.allSettled(queue);
    }

    queueRef.current = [];
    busyRef.current = false;

    const { status, value, reason } = responses.pop();
    if (status === "rejected") {
      setLoading(false);
      errorHandler(reason);
      return;
    }

    setResponse(value);
    setLoading(false);
  }, [errorHandler]);

  const enqueue = useCallback(
    (promise) => {
      if (queueRef.current.length > MAX_QUEUE_LENGTH) {
        throw new Error(
          `Max queue length (${MAX_QUEUE_LENGTH}) exceeded in useQueuedResponse.`
        );
      }
      queueRef.current.push(promise);
      processQueue();
    },
    [queueRef, processQueue]
  );

  return { response, setResponse, loading, enqueue };
}

export function useAsync({ fcn, call = true, init = null, onError = null }) {
  const { response, setResponse, loading, enqueue } = useQueuedResponse({
    init,
    onError,
    call
  });

  const reset = useCallback(() => setResponse(init), [setResponse, init]);

  const wrapper = useCallback(
    (...args) => {
      enqueue(fcn(...args));
    },
    [fcn, enqueue]
  );

  useEffect(() => {
    if (call) wrapper();
  }, [wrapper, call]);

  return [response, wrapper, reset, loading];
}

function useCallAndUnpack(fcn, unpacker = (x) => x) {
  return useCallback(async (...args) => {
    const response = await fcn(...args);
    const unpacked = unpacker(response.obj, response.status);
    return unpacked;
  }, []);
}

export function unpackIOC(ioc) {
  ioc = { ...ioc };
  const {
    id,
    description,
    owner,
    active,
    activeDeployment,
    namingName,
    gitProjectId,
    externalNameId,
    sourceUrl,
    sourceVersion,
    sourceVersionShort,
    operationInProgress,
    alerts
  } = ioc;

  let unpackedIOC = {
    id: id,
    description: description,
    owner: owner,
    activeDeployment: activeDeployment,
    active: active,
    gitProjectId: gitProjectId,
    sourceUrl: sourceUrl,
    sourceVersion: sourceVersion,
    sourceVersionShort: sourceVersionShort,
    namingName: namingName,
    operationInProgress: operationInProgress,
    alerts: alerts,
    externalNameId: externalNameId
  };

  return unpackedIOC;
}

export function unpackIocInfo(ioc) {
  ioc = { ...ioc };
  return ioc;
}

const emptyIocListResponse = {
  totalCount: 0,
  pageNumber: 0,
  limit: 0,
  iocList: []
};

export function unpackIOCList(iocs) {
  let iocArray = iocs.iocList.map((ioc) => unpackIocInfo(ioc));

  let unpackedIOCList = {
    totalCount: iocs.totalCount,
    pageNumber: iocs.pageNumber,
    limit: iocs.limit,
    iocList: iocArray
  };

  return unpackedIOCList;
}

export function useIOCList() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.IOCs.listIocs, unpackIOCList);
  return useAsync({ fcn: method, init: emptyIocListResponse });
}

export function useIOCSearch() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (params) => api.apis.IOCs.listIocs(params),
    unpackIOCList
  );
  return useAsync({ fcn: method, call: false, init: emptyIocListResponse });
}

export function useIOC(id, onError) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.IOCs.getIoc, unpackIOC);
  const boundMethod = useCallback(method.bind(null, { ioc_id: id }), [id]);
  return useAsync({ fcn: boundMethod, onError: onError });
}

export function useCreateIOC(onError) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (body) => api.apis.IOCs.createIoc({}, { requestBody: body }),
    unpackIOC
  );
  return useAsync({ fcn: method, call: false, onError: onError });
}

export function unpackDeployment(deployment) {
  const d = { ...deployment };
  return d;
}

export function unpackDeploymentList(deployments) {
  let deploymentArr = deployments.operations.map((d) => unpackDeployment(d));

  let unpackedDeploymentList = {
    totalCount: deployments.totalCount,
    pageNumber: deployments.pageNumber,
    limit: deployments.limit,
    operationsList: deploymentArr
  };

  return unpackedDeploymentList;
}

export function useDeploymentListForIOC() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (iocId, page, limit) =>
      api.apis.Deployments.listDeployments({
        ioc_id: iocId,
        page: page,
        limit: limit
      }),
    unpackOperationsList
  );
  return useAsync({
    fcn: method,
    call: false,
    init: emptyOperationsListResponse
  });
}

export function unpackCommand(command) {
  const c = { ...command };
  return c;
}

export function unpackCommandList(commandList) {
  let commandArr = commandList.commands.map((c) => unpackCommand(c));

  let unpackedCommandList = {
    totalCount: commandList.totalCount,
    pageNumber: commandList.pageNumber,
    limit: commandList.limit,
    commandList: commandArr
  };

  return unpackedCommandList;
}

export function unpackAllCommandList(input) {
  const { commands: operationsList, ...rest } = input;
  const output = { ...rest, operationsList };
  return output;
}

export function useCommandList() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (iocId, deploymentId, page, limit) =>
      api.apis.Deployments.listCommands({
        ioc_id: iocId,
        deployment_id: deploymentId,
        page: page,
        limit: limit
      }),
    unpackAllCommandList
  );
  return useAsync({
    fcn: method,
    call: false,
    init: emptyOperationsListResponse
  });
}

export function unpackOngoingCommand(command) {
  if (command) {
    return { ...command };
  }
  return null;
}

export function useOngoingCommand(iocId) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    api.apis.Deployments.ongoingCommandByIoc,
    unpackOngoingCommand
  );
  const boundMethod = useCallback(method.bind(null, { ioc_id: iocId }), [
    iocId
  ]);
  return useAsync({ fcn: boundMethod, call: false });
}

export function useDeploymentSearch() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (params) => api.apis.Deployments.listDeployments(params),
    unpackDeploymentList
  );
  return useAsync({
    fcn: method,
    call: false,
    init: emptyOperationsListResponse
  });
}

const emptyOperationsListResponse = {
  totalCount: 0,
  pageNumber: 0,
  limit: 0,
  operationsList: []
};

export function unpackOperationsList(input) {
  const { operations: operationsList, ...rest } = input;
  const output = { ...rest, operationsList };
  return output;
}

export function useOperationsSearch() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (params) => api.apis.Deployments.listOperations(params),
    unpackOperationsList
  );
  return useAsync({
    fcn: method,
    call: false,
    init: emptyOperationsListResponse
  });
}

export function useDeployment(id, onError) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    api.apis.Deployments.fetchDeployment,
    unpackDeployment
  );
  const boundMethod = useCallback(method.bind(null, { deployment_id: id }), [
    id
  ]);
  return useAsync({ fcn: boundMethod, onError: onError });
}

export function useDeploymentById() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (deploymentId) =>
      api.apis.Deployments.fetchDeployment({ deployment_id: deploymentId }),
    unpackDeployment
  );
  return useAsync({ fcn: method, call: false, init: null });
}

export function useCommandById() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (commandId) => api.apis.Deployments.fetchCommand({ command_id: commandId }),
    unpackCommand
  );
  return useAsync({ fcn: method, call: false, init: null });
}

export function unpackDeploymentJob(job) {
  return { ...job };
}

export function useDeploymentJob(awxJobId, onError) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    api.apis.Deployments.fetchDeploymentJobDetails,
    unpackDeploymentJob
  );
  const boundMethod = useCallback(method.bind(null, { awx_job_id: awxJobId }), [
    awxJobId
  ]);
  return useAsync({ fcn: boundMethod, call: true, onError: onError });
}

export function useDeploymentJobById() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (awxJobId) =>
      api.apis.Deployments.fetchDeploymentJobDetails({ awx_job_id: awxJobId }),
    unpackDeploymentJob
  );
  return useAsync({ fcn: method, call: false, init: null });
}

export function unpackCommandJob(job) {
  return { ...job };
}

export function useDeploymentJobLogById() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack((awxJobId) =>
    api.apis.Deployments.fetchDeploymentJobLog({ awx_job_id: awxJobId })
  );
  return useAsync({ fcn: method, call: false, init: null });
}

export function useCommandJob(awxCommandId) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    api.apis.Deployments.fetchCommandDetails,
    unpackCommandJob
  );
  const boundMethod = useCallback(
    method.bind(null, { awx_command_id: awxCommandId }),
    [awxCommandId]
  );
  return useAsync({ fcn: boundMethod, call: true });
}

export function useUpdateAndDeployIoc(id, onError) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (body) =>
      api.apis.IOCs.updateAndDeployIoc({ ioc_id: id }, { requestBody: body }),
    unpackDeployment
  );
  return useAsync({ fcn: method, call: false, onError: onError });
}

export function useCreateUndeployment(id, onError) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (body) =>
      api.apis.IOCs.createUndeployment({ ioc_id: id }, { requestBody: body }),
    unpackDeployment
  );
  return useAsync({ fcn: method, call: false, onError: onError });
}

export function unpackHost(host) {
  return { ...host };
}

export function unpackHostList(hosts) {
  return hosts.csEntryHosts.map((h) => unpackHost(h));
}

export function useHostList() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.Hosts.listHosts, unpackHostList);
  return useAsync({ fcn: method, init: [] });
}

export function useHost(id, onError) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.Hosts.findHostById, unpackHost);
  const boundMethod = useCallback(method.bind(null, { host_csentry_id: id }), [
    id
  ]);
  return useAsync({ fcn: boundMethod, onError: onError });
}

export function useCSentryHost(onError) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (id) => api.apis.Hosts.findCSentryHostById({ host_csentry_id: id }),
    unpackHost
  );
  return useAsync({ fcn: method, call: false, onError: onError });
}

export function useAvailableHost(id) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.Hosts.hostDetail, unpackHost);
  const boundMethod = useCallback(
    method.bind(null, { hostId: id }),
    method,
    id
  );
  return useAsync({ fcn: boundMethod });
}

export function unpackCSEntryHost(host) {
  return { ...host };
}

const emptyHostListResponse = {
  totalCount: 0,
  pageNumber: 0,
  limit: 0,
  hostList: []
};

export function unpackCSEntryHostList(hosts) {
  let hostArr = hosts.csEntryHosts.map((h) => unpackCSEntryHost(h));

  let unpackedHostList = {
    totalCount: hosts.totalCount,
    pageNumber: hosts.pageNumber,
    limit: hosts.limit,
    hostList: hostArr
  };

  return unpackedHostList;
}

export function useCSEntrySearch() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (params) => api.apis.Hosts.listHosts(params),
    unpackCSEntryHostList
  );
  return useAsync({ fcn: method, call: false, init: emptyHostListResponse });
}

const emptyHostIocListResponse = {
  totalCount: 0,
  pageNumber: 0,
  limit: 0,
  deployedList: []
};

export function unpackRecord(record) {
  return { ...record };
}

export function unpackRecordList(records) {
  let recordArray = records.recordList.map((r) => unpackRecord(r));

  let unpackedRecordList = {
    totalCount: records.totalCount,
    pageNumber: records.pageNumber,
    listSize: records.listSize,
    limit: records.limit,
    recordList: recordArray
  };

  return unpackedRecordList;
}

export function useRecordSearch() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (params) => api.apis.Records.findAllRecords(params),
    unpackRecordList
  );
  return useAsync({ fcn: method, call: false, init: emptyRecordListResponse });
}

const emptyRecordListResponse = {
  totalCount: 0,
  pageNumber: 0,
  listSize: 0,
  limit: 0,
  recordList: []
};

export function useRecord(name, onError) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.Records.getRecord, unpackRecord);
  const boundMethod = useCallback(method.bind(null, { name: name }), [name]);
  return useAsync({ fcn: boundMethod, onError: onError });
}

export function unpackHostIOCList(iocs) {
  const deployedArr = iocs.deployedIocs.map((ioc) => unpackIocInfo(ioc));

  let unpackedHostIocList = {
    totalCount: iocs.totalCount,
    pageNumber: iocs.pageNumber,
    limit: iocs.limit,
    deployedList: deployedArr
  };

  return unpackedHostIocList;
}

export function useHostIOCList() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (params) => api.apis.Hosts.findAssociatedIocsByHostId(params),
    unpackHostIOCList
  );
  return useAsync({ fcn: method, call: false, init: emptyHostIocListResponse });
}

export function unpackLogin(loginResponse) {
  return { ...loginResponse };
}

export function useLogin(onError) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (username, password) =>
      api.apis.Authentication.login(
        {},
        {
          requestBody: {
            userName: username,
            password: password
          }
        }
      ),
    unpackLogin
  );
  return useAsync({ fcn: method, call: false, onError: onError });
}

export function unpackUser(user) {
  if (user?.length > 0) {
    return { ...user[0] };
  } else {
    return {};
  }
}

export function useUser() {
  const onError = useCallback((message) => {
    console.warn("useUser error: ", message);
  }, []);

  const [userInfo, getUserInfo, resetUserInfo, userInfoLoading] =
    useUserInfo(onError);
  const [userRoles, getUserRoles, resetUserRoles, userRolesLoading] =
    useUserRoles(onError);
  const [user, setUser] = useState();
  const [loading, setLoading] = useState(true);

  const dataLoading = userInfoLoading || userRolesLoading;

  useEffect(() => {
    // userInfo and user Roles should:
    // - both be defined when logged in
    // - both be undefined when logged out
    const loginStateConsistent = Boolean(userInfo) === Boolean(userRoles);
    if (loginStateConsistent && !dataLoading) {
      setUser({ userInfo, userRoles });
      setLoading(false);
    }
  }, [dataLoading, userInfo, userRoles]);

  const getUser = useCallback(() => {
    setLoading(true);
    getUserInfo();
    getUserRoles();
  }, [getUserInfo, getUserRoles]);

  const resetUser = useCallback(() => {
    setLoading(true);
    resetUserInfo();
    resetUserRoles();
  }, [resetUserInfo, resetUserRoles]);

  return [user, getUser, resetUser, loading];
}

export function useUserInfo(onError) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.Git.infoFromUserName, unpackUser);
  return useAsync({ fcn: method, call: false, onError: onError });
}

export function unpackUserRoles(roles) {
  return roles;
}

export function useUserRoles(onError) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    api.apis.Authentication.getUserRoles,
    unpackUserRoles
  );
  return useAsync({ fcn: method, call: false, onError: onError });
}

export function useLogout() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.Authentication.logout);
  const [, /* value*/ logout /* reset*/, , loading] = useAsync({
    fcn: method,
    call: false
  });
  return [logout, loading];
}

export function useStartIOC(id, onError) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.IOCs.startIoc);
  const boundMethod = useCallback(method.bind(null, { ioc_id: id }), [id]);
  return useAsync({ fcn: boundMethod, call: false, onError: onError });
}

export function useStopIOC(id, onError) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.IOCs.stopIoc);
  const boundMethod = useCallback(method.bind(null, { ioc_id: id }), [id]);
  return useAsync({ fcn: boundMethod, call: false, onError: onError });
}

export function useRenewToken() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.Authentication.tokenRenew);

  const [, /* response*/ renewToken /* reset*/, , loading] = useAsync({
    fcn: method,
    call: false
  });
  return [renewToken, loading];
}

export function unpackLokilog(logData) {
  return { ...logData };
}

export function useLokiSysLog() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (hostName, timeRange) =>
      api.apis.Monitoring.fetchSyslogLines({
        host_name: hostName,
        time_range: timeRange
      }),
    unpackLokilog
  );
  return useAsync({ fcn: method, call: false });
}

export function useLokiProcServLog() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (hostName, iocName, timeRange) =>
      api.apis.Monitoring.fetchProcServLogLines({
        host_name: hostName,
        ioc_name: iocName,
        time_range: timeRange
      }),
    unpackLokilog
  );

  return useAsync({ fcn: method, call: false });
}

export function unpackIocStatus(status) {
  return { ...status };
}

export function useIocStatus() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (iocId) => api.apis.Monitoring.fetchIocStatus({ ioc_id: iocId }),
    unpackIocStatus
  );
  return useAsync({ fcn: method, call: false });
}

// export function useIocStatus(id) {
//   const api = useContext(apiContext);
//   const method = useCallAndUnpack(api.apis.Monitoring.fetchIocStatus, unpackIocStatus);
//   const boundMethod = useCallback(method.bind(null, {ioc_id: id}), [id])
//   return useAsync({ fcn: boundMethod});
// }

export function unpackStatistics(statistics) {
  return { ...statistics };
}

export function useStatistics() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    api.apis.Statistics.generalStatistics,
    unpackStatistics
  );

  return useAsync({ fcn: method, call: false });
}

export function unpackNaming(naming) {
  return { ...naming };
}

export function unpackNamingList(naming) {
  return naming.map((h) => unpackNaming(h));
}

export function useNamingNames() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (iocName) => api.apis.Naming.fetchIOCByName({ ioc_name: iocName }),
    unpackNamingList
  );

  return useAsync({ fcn: method, call: false, init: [] });
}

export function unpackUserList(users) {
  return users.toString().split(",");
}

export function useUserNames() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (query) => api.apis.Authentication.getToolUsers({ query: query }),
    unpackUserList
  );

  return useAsync({ fcn: method, call: false, init: [] });
}

export function unpackDeploymentStatistics(statistics) {
  return { ...statistics };
}

export function useDeploymentStatistics() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    api.apis.Statistics.deploymentStatistics,
    unpackDeploymentStatistics
  );

  return useAsync({ fcn: method, call: false });
}

export function unpackGitReference(reference) {
  return { ...reference };
}

export function unpackTagAndCommitIdList(tagList) {
  return tagList.map((t) => unpackGitReference(t));
}

export function useTagsAndCommitIds(onError) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (gitProjectId, reference, includeAllReference, searchMethod) =>
      api.apis.Git.listTagsAndCommitIds({
        project_id: gitProjectId,
        reference: reference,
        include_all_reference: includeAllReference,
        search_method: searchMethod
      }),
    unpackTagAndCommitIdList
  );
  return useAsync({ fcn: method, call: false, onError: onError, init: [] });
}

export function unpackIOCStatistics(statistics) {
  return { ...statistics };
}

export function useIOCStatistics() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    api.apis.Statistics.iocStatistics,
    unpackIOCStatistics
  );

  return useAsync({ fcn: method, call: false });
}

export function unpackCurrentlyActiveIOCS(statistics) {
  return { ...statistics };
}

export function useCurrentlyActiveIOCs() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    api.apis.Statistics.currentlyActiveIocs,
    unpackCurrentlyActiveIOCS
  );

  return useAsync({ fcn: method, call: false });
}

export function unpackActiveIOCHistory(statistics) {
  return { ...statistics };
}

export function useActiveIOCHistory() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    api.apis.Statistics.activeIocHistory,
    unpackActiveIOCHistory
  );

  return useAsync({ fcn: method, call: false });
}

export function unpackDeployedIOCStat(statistics) {
  return { ...statistics };
}

export function useDeployedIOCStatistics() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    api.apis.Statistics.iocDeploymentHistory,
    unpackDeployedIOCStat
  );

  return useAsync({ fcn: method, call: false });
}

export function unpackGitProject(project) {
  return { ...project };
}

export function unpackAllowedGitProjects(projectList) {
  return projectList.map((p) => unpackGitProject(p));
}

export function useAllowedGitProjects() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (repoUrl) => api.apis.Git.listProjects(),
    unpackAllowedGitProjects
  );
  return useAsync({ fcn: method, call: false, init: [] });
}

export function useDeleteIOC(id, onError) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    api.apis.IOCs.deleteIoc,
    unpackDeleteIocResponse
  );
  const boundMethod = useCallback(method.bind(null, { ioc_id: id }), [id]);
  return useAsync({ fcn: boundMethod, call: false, onError: onError });
}

export function unpackDeleteIocResponse(_, status) {
  if (status === 204) {
    return "Successful delete";
  }
}

export function useUpdateIoc(id, onError) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (body) => api.apis.IOCs.updateIoc({ ioc_id: id }, { requestBody: body }),
    unpackUpdateIoc
  );
  return useAsync({ fcn: method, call: false, onError: onError });
}

export function unpackUpdateIoc(updateIoc) {
  const d = { ...updateIoc };
  return d;
}

export function unpackUndeployInDb(deployment) {
  return { ...deployment };
}

export function useUndeployInDb(id, onError) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    api.apis.Deployments.unDeployInDb,
    unpackUndeployInDb
  );
  const boundMethod = useCallback(method.bind(null, { ioc_id: id }), [id]);
  return useAsync({ fcn: boundMethod, call: false, onError: onError });
}

export function unpackUpdateActiveDeploymentHost(ioc) {
  return { ...ioc };
}

export function useUpdateActiveDeploymentHost(id, onError) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (body) =>
      api.apis.IOCs.updateActiveDeploymentHost(
        { ioc_id: id },
        { requestBody: body }
      ),
    unpackUpdateActiveDeploymentHost
  );
  return useAsync({ fcn: method, call: false, onError: onError });
}

export function useAnnouncements() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    api.apis.Broadcasts.fetchAnnouncements,
    unpackAnnouncement
  );

  return useAsync({ fcn: method, call: true });
}

export function unpackAnnouncement(announcements) {
  return { ...announcements };
}

export function usePersonalStatistics() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    api.apis.Statistics.personalStatistics,
    unpackPersonalStatistics
  );

  return useAsync({ fcn: method, call: true });
}

export function unpackPersonalStatistics(statistics) {
  return { ...statistics };
}

export function useOwnIocsWithAlarms() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.IOCs.listOwnIocsWithAlarms);

  return useAsync({ fcn: method, call: true, init: [] });
}
